
# Author: Andre Minoche, a.minoche@garvan.org.au

use FindBin;                 # locate this script
use lib "$FindBin::Bin/../../perlib";  # use the parent directory
print STDERR "local perl lib: ".$FindBin::Bin."/../../perlib\n";

use My::VCF2;
use POSIX;
use Bio::DB::BigWig "binMean";
use File::Path qw(make_path remove_tree);

$projectDir=shift(@ARGV);
$inRef=shift(@ARGV);
$fileExt=shift(@ARGV);
$cSample=shift(@ARGV);
$nameStemJoinF=shift(@ARGV);

print STDERR "projectDir: $projectDir\n";
print STDERR "inRef: $inRef\n";
print STDERR "fileExt: $fileExt\n";
print STDERR "sample: $cSample\n";

#read in chr2len
open(IN1, "<$inRef.fai") || die "can not open $inRef";
while(<IN1>){ chomp; @_=split("\t",$_); $chr2len{$_[0]}=$_[1]; }close(IN1);


########################### define expected values, and range from NA12878

%stat;
@categories = ("bamfile","testRegion","PcNotPropPair","PcMapDiffChr","covStdev","concInsSizeMin","concInsSizeMax","insMean","insSD","inputDiscPairs","inputSplitReads","avgCovAutosomes","avgCovX","avgCovY","avgCovMT","avgYOrig");

@SvStatCats = ("svStatHigh","svStatPassHigh","svStatLow","svStatCNVs","svStatCNVLoss","svStatCNVGain","svStatNonCNV","svStatToolLumpy","svStatToolCNVnator","svStatToolBoth","svStatGene","svStatRare","svStatRarePassHigh","svStatRarePassHighGene","svStatRarePassHighGeneCNV");

@categories = (@categories,@SvStatCats); # add SV type categories

#################### per sample

	
	$doBamStats=1;
	
	if ($doBamStats){
		if(-e "$projectDir/alignments/$cSample/$cSample.bam" ){ $cBam="$projectDir/alignments/$cSample/$cSample.bam"; }
		elsif(-e "$projectDir/alignments/$cSample/$cSample.IR.bam" ){ $cBam="$projectDir/alignments/$cSample/$cSample.IR.bam"; }
		else{print STDERR "can not locate bam file\n"; exit 1; }
	
		print STDERR "#### current sample: $cSample \n";
		print STDERR "# use bam file: $cBam \n"; $stat{$cSample}{bamfile}=$cBam;
		############### from bam compute 
		# 
		# - coverage, STDEV
		# 
		# 
		# - % badmates

		# - insertsize modal, mean, stdev, plot
		$countDiffrChr=0;
		$countPcNotPropPair=0;
		$countAllPairs=0;
		undef @tmpAIns;
		make_path($projectDir."/SVs/qc/insertSizes") if (! -d $projectDir."/SVs/qc/insertSizes");
		open(OUTINS, ">$projectDir/SVs/qc/insertSizes/$cSample.txt") || die "can not read bam file";
	
		open(INB, "samtools view $cBam 1:20000001-30000000 | ") || die "can not read bam file"; 
		# ST-E00141:48:H0ATBALXX:1:1220:8105:62471        163     1       19999858        60      150M    =       20000040        332     TCCAGAGTCTGTCTTGT  <AFFFJJF  NM:i:1  MD:Z:149A0      AS:i:149        XS:i:70 MC:Z:150M       MQ:i:60 RG:Z:H0ATBALXX_1
		while(<INB>){
		
			chomp; @_=split("\t",$_);
		
			next if (!($_[1] & 64)); # next if not first in pair
		
			$countAllPairs++;
			
			if($_[1] & 2){  if($_[6] eq "="){ print OUTINS abs($_[8])."\n"; push @tmpAIns, abs($_[8])  if abs($_[8]) < 1500;  }   }    
		
			if(!($_[1] & 2)){  $countPcNotPropPair++; }
			if($_[6] ne "="){  $countDiffrChr++;  }
		


		
		}
		close(INB);
	
		close(OUTINS);
	
		print STDERR " of $countAllPairs read pairs in region 1:20000001-30000000\n";
		$PcNotPropPair=round($countPcNotPropPair/$countAllPairs*100,3);
		$PcMapDiffChr=round($countDiffrChr/$countAllPairs*100,3);
		print STDERR " % not proper pair: $PcNotPropPair\n";
		print STDERR " % mapping different Chr: $PcMapDiffChr\n";
	
		$stat{$cSample}{testRegion}="1:20000001-30000000";
		$stat{$cSample}{PcNotPropPair}=$PcNotPropPair;
		$stat{$cSample}{PcMapDiffChr}=$PcMapDiffChr;
	
		$stat{$cSample}{insMean}=round(average(\@tmpAIns),0);
		$stat{$cSample}{insSD}=round(stdev(tmpAIns),0);
	
	
	
		####### stdev
	
		undef @tmpArr;
		open(INB, "samtools depth -r 1:20000001-30000000 $cBam | ") || die "can not read bam file"; 
		# ST-E00141:48:H0ATBALXX:1:1220:8105:62471        163     1       19999858        60      150M    =       20000040        332     TCCAGAGTCTGTCTTGT  <AFFFJJF  NM:i:1  MD:Z:149A0      AS:i:149        XS:i:70 MC:Z:150M       MQ:i:60 RG:Z:H0ATBALXX_1
		while(<INB>){
			chomp; @_=split("\t",$_);
			push @tmpArr, $_[2];
		}
		close(INB);

		$covStdev=round(stdev(\@tmpArr),1);
		print STDERR " covStdev: $covStdev\n";
		$stat{$cSample}{covStdev}=$covStdev;
		undef @tmpArr;
	
	}
	
	############### from STDERR output 
	# 
	# - select used insS cutoffs

	if ( -e "$projectDir/SVs/$cSample/lumpy/sh/lumpy.preproc.$cSample.e"){
		$insSCutOffPERaw=`grep "insert cutoff low" $projectDir/SVs/$cSample/lumpy/sh/lumpy.preproc.$cSample.e`;
	}else{
		$insSCutOffPERaw=`grep "insert cutoff low" $projectDir/SVs/$cSample/lumpy/$cSample.lumpy.metrics`;
	}
	
	if($insSCutOffPERaw =~ /insert cutoff low ([0-9]+) \(to obtain [0-9]+ rpm\), high ([0-9]+)/ ){  $concPErange="$1-$2";   }
	print STDERR " used insertsize range for concordant pairs: $concPErange\n";
	($stat{$cSample}{concInsSizeMin},$stat{$cSample}{concInsSizeMax})=split(/\-/,$concPErange);

	############### SE,PR
	# 
	# - input split reads discordant pairs
	
	if(-e "$projectDir/SVs/$cSample/lumpy/$cSample.discordants.bam"){
		$inputPEs=`samtools view -c $projectDir/SVs/$cSample/lumpy/$cSample.discordants.bam`; chomp($inputPEs);
		print STDERR "# use discordants file $projectDir/SVs/$cSample/lumpy/$cSample.discordants.bam  \n";
	}else{ # in case of multiple lanes per sample
		foreach $cFile (glob("$projectDir/SVs/$cSample/lumpy/$cSample\_*.discordants.bam")){
			print STDERR "# use discordants file $cFile  \n";
			$inputPEsX=`samtools view -c $cFile`; chomp($inputPEsX);
			$inputPEs+=$inputPEsX;
		}
	}
	$inputSRs=`samtools view -c $projectDir/SVs/$cSample/lumpy/$cSample.splitters.f.bam`; chomp($inputSRs);
	
	print STDERR " number input discordant pairs: $inputPEs\n";
	print STDERR " number input split reads: $inputSRs\n";
	
	$stat{$cSample}{inputDiscPairs}=$inputPEs; 
	$stat{$cSample}{inputSplitReads}=$inputSRs; 


	############### chromosome wide plot
	# 
	# - 1Mb window, with STDEV from control samples as error bars 


	print STDERR "open the bw files...\n";
	############################################################
	our %bwObj;


	foreach $cT (("mq","q0","q20")){			
# 		$cQBW=$jobfs."/".$cSample.".$cT.bw";
		$cQBW=$projectDir."/alignments/$cSample/bw/".$cSample.".$cT.bw";
		die " $cQBW ! exists" if (! -f $cQBW);
		$bwObj{$cSample}{$cT} = Bio::DB::BigWig->new(-bigwig=>$cQBW, -fasta=>$inRef);
	}	
	print STDERR "# sample: $cSample, bw found\n";


	print STDERR "determine the average coverage per sample acrosse >MQ50 regions...\n";
	############################################################
	%meanCov;

	$sumCov=0;
	for(1..22){
		$sumCov+=getCov($_,20000001,30000000,"q0",58,$cSample);
	}
	foreach $cChr ((keys %chr2len,"A")){ $meanCov{$cChr}{$cSample}=sprintf("%.2f",($sumCov)/22); }

	# determine X and Y coverage and round to 0, 0.5, 1, 1.5, 2, ... times average coverage
	$meanCov{X}{$cSample}=round((getCov("X",1,$chr2len{X},"q0",58,$cSample)/$meanCov{A}{$cSample})/0.5,0)*0.5*$meanCov{A}{$cSample};
	$meanCov{Y}{$cSample}=round( (getCov("Y",6641419,10079253,"q0",0,$cSample)+getCov("Y",13800704,23668908,"q0",0,$cSample))/2, 5 ); # two mq 60 region one of each Y arm 
	$meanCovYOrig=$meanCov{Y}{$cSample};	
	$meanCov{Y}{$cSample}=0 if $meanCov{Y}{$cSample} == (-1); # if -1 then because it is not covered	
	$meanCov{Y}{$cSample}=round(($meanCov{Y}{$cSample}/$meanCov{A}{$cSample})/0.5,0)*0.5*$meanCov{A}{$cSample};

	# MT coverage
	$meanCov{MT}{$cSample}=sprintf("%.2f",(getCov("MT",1,$chr2len{MT},"q0",58,$cSample)));	

	print STDERR "average coverage $cSample autosome:".$meanCov{A}{$cSample}.", X:".$meanCov{X}{$cSample}.", Y:".$meanCov{Y}{$cSample}." Yorig: $meanCovYOrig, MT:".$meanCov{MT}{$cSample}."\n";
	$stat{$cSample}{avgCovAutosomes}=$meanCov{A}{$cSample}; 
	$stat{$cSample}{avgCovX}=$meanCov{X}{$cSample}; 
	$stat{$cSample}{avgCovY}=$meanCov{Y}{$cSample}; 
	$stat{$cSample}{avgCovMT}=$meanCov{MT}{$cSample}; 
	$stat{$cSample}{avgYOrig}=$meanCovYOrig; 

		
	######### 1 dot per MB,  output chr:value,value,value
	make_path($projectDir."/SVs/qc/chromView") if (! -d $projectDir."/SVs/qc/chromView");
	open(OUTCHRCOV, ">$projectDir/SVs/qc/chromView/$cSample.txt") || die "can not read bam file";
	
	$num{X}=round(round($meanCov{X}{$cSample}/$meanCov{A}{$cSample},1)*2,0);
	$num{Y}=round(round($meanCov{Y}{$cSample}/$meanCov{A}{$cSample},1)*2,0);
	
	print STDERR "numX: ".$num{X}.", numY: ".$num{Y}.", ".("X"  x  $num{X} ).", ".("Y"  x  $num{Y} )."\n";
		
	for $cChr ((1..22),"X","Y","MT"){
		$qBins=ceil($chr2len{$cChr}/1000000);
		undef @aCov; undef(@covVals);
		
		$cDivCov=$meanCov{$cChr}{$cSample};
		$cChr2=$cChr;
		
		if( $cChr eq "X"  or $cChr eq "Y" ){
			# new logic: if XX devide by autosomal average
			#            if X,Y0,X0    devide by 1/2 autosomal average
			$cDivCov=$meanCov{A}{$cSample}*0.5 if $num{$cChr}<2;	
			$cChr2=($num{$cChr}>=1)? ($cChr  x  $num{$cChr} ):$cChr."0"; # XX or X or Y or X0 or Yo
		}
		
		@aCov = $bwObj{$cSample}{"q0"}->features(-seq_id=>$cChr,-start=>1,-end=>$chr2len{$cChr},-type=>"bin:$qBins");
		
		for ($i=0; $i<=$#aCov;$i++){
		  my ($cSt,$cEn,$cCov)  = ($aCov[$i]->start,$aCov[$i]->end,$aCov[$i]->mean);
		  
		  $cCovVals=round($cCov/$cDivCov,2);
		  $cCovVals+=1 if  ($cChr eq "X"  or $cChr eq "Y" ) and $num{$cChr} == 0 and $cCovVals>=0;
		  push @covVals, $cCovVals;
		}
		
		print OUTCHRCOV "$cChr2\t".join("\t",@covVals)."\n";
	}
	close(OUTCHRCOV);

	############### number variants
	# 
	# type: CNVs, LOSS, GAIN, non CNV
	# 
	# tool: CNVnator, Lumpy, merged calls
	# 
	# PASS:  LOW, PASS, HIGH
	# 
	# hitting genes
	# 
	# RARE variants
	# 
	# RARE PASS HIGH
	# 
	# RARE PASS HIGH hitting genes
	# 
	# list of these variants # with R
	# DEL, DUP, other, sorted by size or evidence
	# size distribution, expected in grey
	# 

	

	$inVCF="$projectDir/SVs/$nameStemJoinF/SV-CNV$fileExt.vcf";
	print STDERR "inVCF $inVCF\n";
	my $V=My::VCF2->new($inVCF);

	while (my $V=$V->next_line){
		
		next if exists($$V{INFO}{SECONDARY});
		
		$stat{$cSample}{svStatHigh}++ if $$V{GTS}{FT}{$cSample} eq "HIGH";
		$stat{$cSample}{svStatPassHigh}++ if $$V{GTS}{FT}{$cSample} eq "PASS" or $$V{GTS}{FT}{$cSample} eq "HIGH";
		$stat{$cSample}{svStatLow}++ if $$V{GTS}{FT}{$cSample} eq "LOW";
		
		next if $$V{GTS}{FT}{$cSample} ne "PASS" and $$V{GTS}{FT}{$cSample} ne "HIGH";
		
		if ( $$V{INFO}{CNV}==1 or ( exists($$V{INFO}{CNEUTR}) and $$V{INFO}{CNEUTR}==0 ) ){
			$stat{$cSample}{svStatCNVs}++;
			if ($$V{INFO}{SVTYPE} eq "DEL"){
				$stat{$cSample}{svStatCNVLoss}++;
			}elsif($$V{INFO}{SVTYPE} eq "DUP"){
				$stat{$cSample}{svStatCNVGain}++;
			}elsif($$V{GTS}{DRA}{$cSample} < 1){
				$stat{$cSample}{svStatCNVLoss}++;
			}else{
				$stat{$cSample}{svStatCNVGain}++;
			}
		}else{  $stat{$cSample}{svStatNonCNV}++; } # next; }
				
		$stat{$cSample}{svStatToolLumpy}++ if $$V{INFO}{TOOL} eq "Lumpy";
		$stat{$cSample}{svStatToolCNVnator}++ if $$V{INFO}{TOOL} eq "CNVnator";
		$stat{$cSample}{svStatToolBoth}++ if $$V{INFO}{TOOL} eq "Lumpy,CNVnator";			
			
		$stat{$cSample}{svStatGene}++ if $$V{INFO}{NUMG}>0;
				
		$stat{$cSample}{svStatRare}++ if $$V{GTS}{RARE}{$cSample} ==1;
		
		$stat{$cSample}{svStatRarePassHighGene}++ if $$V{GTS}{RARE}{$cSample} ==1 and  $$V{INFO}{NUMG}>0;
		$stat{$cSample}{svStatRarePassHighGeneCNV}++ if $$V{GTS}{RARE}{$cSample} ==1 and  $$V{INFO}{NUMG}>0 and ( $$V{INFO}{CNV}==1 or ( exists($$V{INFO}{CNEUTR}) and $$V{INFO}{CNEUTR}==0 ) );
		
		$stat{$cSample}{svStatRarePassHigh}++ if $$V{GTS}{RARE}{$cSample} ==1 ;

	}
	

	


	############### sensitivity with NA12878 gold standard
	# 




	############### reprod CNVs with NA12878
	# 
	
	# prepare the 







############### write the variables for each sample into a table to be read with R

open(OUTTAB, ">$projectDir/SVs/qc/$cSample.stat.tab") || die "can not read bam file";
print OUTTAB join("\t",("Categroy",($cSample)))."\n";
foreach $cCat (@categories){

	
	
	print OUTTAB join("\t",($cCat, map { (exists($stat{$_}{$cCat}))? $stat{$_}{$cCat}:0 } ($cSample) ))."\n";
}
close(OUTTAB);


######################################################################  function


sub getCov{
	my ($qChr,$qSt,$qEn,$covQ,$qMQ,$cSample)=@_;
	($qSt,$qEn)=($qEn,$qSt) if $qSt>$qEn;
	my $qLen=($qEn-$qSt+1);
	my ($sumCov,$fBins)=(0,0);
    my $qBins=100;

    $qBins=$qLen if ($qLen/$qBins)<1;
     
	undef @aCov; undef @aMQ;
	@aCov = $bwObj{$cSample}{$covQ}->features(-seq_id=>$qChr,-start=>$qSt,-end=>$qEn,-type=>"bin:$qBins");
	@aMQ = $bwObj{$cSample}{"mq"}->features(-seq_id=>$qChr,-start=>$qSt,-end=>$qEn,-type=>"bin:$qBins") if $qMQ>0;	

# 	print "$qChr,$qSt,$qEn,$qMQ,$cSample,$qLen, qMQ: $qMQ, bins: $qBins\n";
    
	for ($i=0; $i<=$#aCov;$i++){		
	  my ($cSt,$cEn,$cCov,$cMQ)  = ($aCov[$i]->start,$aCov[$i]->end,$aCov[$i]->mean,0);
	  $cMQ=($aMQ[$i]->mean) if $qMQ>0;
	  my $cLen = ($cEn-$cSt+1); 
# 	  print "c: $cSt,$cEn,$cLen: $cCov,$cMQ ".($cMQ<$qMQ or $cCov<0)."\n";
	  next if $qMQ>0 and $cMQ<$qMQ;
	  next if $cCov<0;	  
	  $sumCov+=$cCov; $fBins++;
	}
# 	print "f: $fBins ".sprintf("%.1f",$sumCov/($fBins+0.0000001))."\n";
	
	if($fBins==0 or $fBins/$qBins<0.33333){ return (-1) }
	else{ return sprintf("%.2f",$sumCov/$fBins) }
}





sub round { my $number = shift || 0; my $dec = 10 ** (shift || 0); return int( $dec * $number + .5 * ($number <=> 0)) / $dec;}




sub stdev{
        my($data) = @_;
        if(@$data == 1){
                return 0;
        }
        my $average = &average($data);
        my $sqtotal = 0;
        foreach(@$data) {
                $sqtotal += ($average-$_) ** 2;
        }
        my $std = ($sqtotal / (@$data-1)) ** 0.5;
        return $std;
}

sub average{
        my($data) = @_;
        if (not @$data) {
                die("Empty array");
        }
        my $total = 0;
        foreach (@$data) {
                $total += $_;
        }
        my $average = $total / @$data;
        return $average;
}




































